#!/usr/bin/env node
import * as __WEBPACK_EXTERNAL_MODULE_path__ from "path";
import * as __WEBPACK_EXTERNAL_MODULE_fs__ from "fs";
import * as __WEBPACK_EXTERNAL_MODULE_child_process__ from "child_process";
import * as __WEBPACK_EXTERNAL_MODULE_url__ from "url";
import * as __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__ from "@clack/prompts";
import * as __WEBPACK_EXTERNAL_MODULE_minimist__ from "minimist";
import * as __WEBPACK_EXTERNAL_MODULE_rslog__ from "rslog";
import * as __WEBPACK_EXTERNAL_MODULE_glob__ from "glob";
import * as __WEBPACK_EXTERNAL_MODULE_fs_extra_ce68a66b__ from "fs-extra";
import * as __WEBPACK_EXTERNAL_MODULE_handlebars__ from "handlebars";
const IMAGE_EXT_LIST = [
    '.jpg',
    '.jpeg',
    '.png',
    '.gif',
    '.bmp',
    '.ico',
    '.icon',
    '.mpt',
    '.psd',
    '.wmf'
];
const FS_RESOURCE = '_mf_fs_resource';
class FsResource {
    constructor(filePath, resourceKey){
        this._type = FS_RESOURCE;
        this.filePath = filePath;
        this.resourceKey = resourceKey;
    }
    async value() {
        const resourceFileExt = __WEBPACK_EXTERNAL_MODULE_path__["default"].extname(this.filePath);
        if (IMAGE_EXT_LIST.includes(resourceFileExt)) {
            const buffer = await __WEBPACK_EXTERNAL_MODULE_fs_extra_ce68a66b__["default"].readFile(__WEBPACK_EXTERNAL_MODULE_path__["default"].resolve(this.filePath));
            return {
                content: buffer
            };
        }
        const text = await __WEBPACK_EXTERNAL_MODULE_fs_extra_ce68a66b__["default"].readFile(__WEBPACK_EXTERNAL_MODULE_path__["default"].resolve(this.filePath), 'utf8');
        return {
            content: text
        };
    }
}
const promisifyGlob = (pattern, options)=>new Promise((resolve, reject)=>{
        (0, __WEBPACK_EXTERNAL_MODULE_glob__["default"])(pattern, options, (err, files)=>null === err ? resolve(files) : reject(err));
    });
class FsMaterial {
    constructor(basePath){
        this.basePath = basePath;
    }
    get(resourceKey) {
        return new FsResource(__WEBPACK_EXTERNAL_MODULE_path__["default"].resolve(this.basePath, resourceKey), resourceKey);
    }
    async find(globStr, options) {
        const matches = await promisifyGlob(globStr, {
            cwd: __WEBPACK_EXTERNAL_MODULE_path__["default"].resolve(this.basePath),
            nodir: options?.nodir,
            dot: options?.dot,
            ignore: options?.ignore
        });
        return matches.reduce((pre, cur)=>{
            pre[cur] = new FsResource(__WEBPACK_EXTERNAL_MODULE_path__["default"].resolve(this.basePath, cur), cur);
            return pre;
        }, {});
    }
}
function renderString(template, fullData) {
    return __WEBPACK_EXTERNAL_MODULE_handlebars__["default"].compile(template)(fullData) || '';
}
async function outputFs(file, content, outputPath, options) {
    const filePath = __WEBPACK_EXTERNAL_MODULE_path__["default"].resolve(outputPath, file.toString());
    await __WEBPACK_EXTERNAL_MODULE_fs_extra_ce68a66b__["default"].mkdirp(__WEBPACK_EXTERNAL_MODULE_path__["default"].dirname(filePath));
    await __WEBPACK_EXTERNAL_MODULE_fs_extra_ce68a66b__["default"].writeFile(filePath, content, options);
}
class HandlebarsAPI {
    async renderTemplate(templateResource, target, outputFilePath, parameters = {}) {
        if (templateResource._type !== FS_RESOURCE) throw new Error('resource not match');
        const resourceValue = await templateResource.value();
        if ('string' != typeof resourceValue.content) throw new Error(`resource.value is not string, resourceValue=${resourceValue}`);
        await outputFs(target, templateResource.resourceKey.endsWith('.handlebars') ? renderString(resourceValue.content, parameters) : resourceValue.content, outputFilePath, {
            encoding: 'utf-8'
        });
    }
    async renderTemplateDir(material, findGlob, target, outputFilePath, options) {
        const resourceMap = await material.find(findGlob, {
            nodir: true,
            ...options
        });
        await Promise.all(Object.keys(resourceMap).map(async (resourceKey)=>{
            await this.renderTemplate(material.get(resourceKey), target(resourceKey), outputFilePath, options?.parameters);
        }));
    }
}
const create_filename = (0, __WEBPACK_EXTERNAL_MODULE_url__.fileURLToPath)(import.meta.url);
const create_dirname = __WEBPACK_EXTERNAL_MODULE_path__["default"].dirname(create_filename);
const packageDir = __WEBPACK_EXTERNAL_MODULE_path__["default"].resolve(create_dirname, '..');
const OTHER_TYPE = {
    zephyr: {
        label: 'zephyr',
        packageName: 'zephyr-apps@latest',
        hint: 'create zephyr-apps@latest'
    }
};
function upperFirst(str) {
    return str.charAt(0).toUpperCase() + str.slice(1);
}
function logHelpMessage(name, templates) {
    __WEBPACK_EXTERNAL_MODULE_rslog__.logger.log(`
   Usage: create-${name} [options]

   Options:

     -h, --help       display help for command
     -d, --dir        create project in specified directory
     -t, --template   specify the template to use
     -n, --name       specify the mf name
     -r, --role       specify the mf role type: provider or consumer
     --override       override files in target directory

   Templates:
     ${templates.filter((t)=>!t.startsWith('zephyr')).join(', ')}
`);
}
function pkgFromUserAgent(userAgent) {
    if (!userAgent) return;
    const pkgSpec = userAgent.split(' ')[0];
    const pkgSpecArr = pkgSpec.split('/');
    return {
        name: pkgSpecArr[0],
        version: pkgSpecArr[1]
    };
}
function cancelAndExit() {
    (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.cancel)('Operation cancelled.');
    process.exit(0);
}
function checkCancel(value) {
    if ((0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.isCancel)(value)) cancelAndExit();
    return value;
}
function formatProjectName(input) {
    const formatted = input.trim().replace(/\/+$/g, '');
    return {
        packageName: input,
        targetDir: formatted
    };
}
function isEmptyDir(path) {
    const files = __WEBPACK_EXTERNAL_MODULE_fs__["default"].readdirSync(path);
    return 0 === files.length || 1 === files.length && '.git' === files[0];
}
async function getAppTemplateName({ roleType, framework }, { template }) {
    if (template) return `${template}-ts`;
    return `${roleType}-${framework}-ts`;
}
async function getLibTemplateName({ template, role }) {
    if (template) return `${template}-ts`;
    const templateName = checkCancel(await (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.select)({
        message: 'Select template',
        options: [
            {
                value: 'rslib',
                label: 'Rslib'
            }
        ]
    }));
    const tools = checkCancel(await (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.multiselect)({
        message: 'Select development tools (Use <space> to select, <enter> to continue)',
        required: false,
        options: [
            {
                value: 'storybook',
                label: 'Storybook'
            }
        ].filter(Boolean)
    }));
    const roleType = role || "provider";
    if (!tools || !Object.keys(tools).length) return `${roleType}-${templateName}-ts`;
    return `${roleType}-${templateName}-${tools[0]}-ts`;
}
function getTemplateName({ projectType, roleType, framework }, args) {
    if ('app' === projectType) return getAppTemplateName({
        roleType,
        framework
    }, args);
    return getLibTemplateName(args);
}
function getTemplateDir(templateName) {
    return `templates/${templateName}/`;
}
async function getProjectType(template) {
    if (!template) return checkCancel(await (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.select)({
        message: 'Please select the type of project you want to create:',
        options: [
            {
                value: "app",
                label: 'Application'
            },
            {
                value: "lib",
                label: 'Lib'
            },
            {
                value: "zephyr",
                label: 'Zephyr Powered (Learn more at https://zephyr-cloud.io)'
            }
        ]
    }));
    if (template.includes('lib')) return "lib";
    return "app";
}
async function forgeTemplate({ projectType, argv, templateParameters, distFolder }) {
    let framework = 'modern';
    let roleType = "provider";
    if (!argv?.template && 'app' === projectType) {
        framework = checkCancel(await (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.select)({
            message: 'Select template',
            options: [
                {
                    value: 'modern',
                    label: 'Modern.js Framework'
                },
                {
                    value: 'rsbuild',
                    label: 'Rsbuild'
                }
            ]
        }));
        roleType = argv.role || checkCancel(await (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.select)({
            message: 'Please select the role of project you want to create:',
            initialValue: 'provider',
            options: [
                {
                    value: 'consumer',
                    label: 'Consumer'
                },
                {
                    value: 'provider',
                    label: 'Provider'
                }
            ]
        }));
    }
    const templateName = await getTemplateName({
        projectType,
        framework,
        roleType
    }, argv);
    const material = new FsMaterial(packageDir);
    const renderTemplate = async (templateDir)=>{
        const templatePattern = `${templateDir}**/*`;
        const resourceMap = await material.find(templatePattern, {
            nodir: true,
            dot: true
        });
        const parameters = {
            ...templateParameters
        };
        await Promise.all(Object.keys(resourceMap).map(async (resourceKey)=>{
            const target = resourceKey.replace(templateDir, "").replace('.handlebars', "");
            const handlebarsAPI = new HandlebarsAPI();
            await handlebarsAPI.renderTemplate(material.get(resourceKey), target, distFolder, {
                ...parameters
            });
        }));
    };
    const templateDir = getTemplateDir(templateName);
    let commonTemplateDir = '';
    if ("lib" === projectType) commonTemplateDir = 'templates/lib-common/';
    else if ('app' === projectType) commonTemplateDir = `templates/${framework}-common/`;
    if (commonTemplateDir) await renderTemplate(commonTemplateDir);
    await renderTemplate(templateDir);
}
async function create({ name, templates }) {
    const argv = (0, __WEBPACK_EXTERNAL_MODULE_minimist__["default"])(process.argv.slice(2), {
        alias: {
            h: 'help',
            d: 'dir',
            t: 'template',
            n: 'name',
            r: 'role'
        }
    });
    console.log('');
    __WEBPACK_EXTERNAL_MODULE_rslog__.logger.greet(`◆  Create ${upperFirst(name)} Project`);
    if (argv.help) {
        logHelpMessage(name, templates);
        return;
    }
    const cwd = process.cwd();
    const pkgInfo = pkgFromUserAgent(process.env['npm_config_user_agent']);
    const pkgManager = pkgInfo ? pkgInfo.name : 'npm';
    const mfVersion = "0.11.2";
    argv.template = templates.includes(argv.template || '') ? argv.template : void 0;
    const projectType = await getProjectType(argv.template);
    if ("zephyr" === projectType) {
        const zephyrPackage = OTHER_TYPE['zephyr'].packageName;
        const zephyrCommand = `${pkgManager} create ${zephyrPackage}`;
        (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.note)(`Running: ${zephyrCommand}`, 'Launching Zephyr setup');
        (0, __WEBPACK_EXTERNAL_MODULE_child_process__.spawnSync)(pkgManager, [
            'create',
            zephyrPackage
        ], {
            stdio: 'inherit',
            shell: true
        });
        (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.outro)('Done.');
        return;
    }
    const mfName = argv.name || checkCancel(await (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.text)({
        message: 'Please input Module Federation name:',
        placeholder: 'mf_project_name',
        defaultValue: 'mf_project_name',
        validate (value) {
            if (0 === value.length) return 'Name is required';
        }
    }));
    const { targetDir } = formatProjectName(__WEBPACK_EXTERNAL_MODULE_path__["default"].join(argv.dir || mfName));
    const distFolder = __WEBPACK_EXTERNAL_MODULE_path__["default"].isAbsolute(targetDir) ? targetDir : __WEBPACK_EXTERNAL_MODULE_path__["default"].join(cwd, targetDir);
    if (!argv.override && __WEBPACK_EXTERNAL_MODULE_fs__["default"].existsSync(distFolder) && !isEmptyDir(distFolder)) {
        const option = checkCancel(await (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.select)({
            message: `"${targetDir}" is not empty, please choose:`,
            options: [
                {
                    value: 'yes',
                    label: 'Continue and override files'
                },
                {
                    value: 'no',
                    label: 'Cancel operation'
                }
            ]
        }));
        if ('no' === option) cancelAndExit();
    }
    await forgeTemplate({
        projectType,
        argv,
        templateParameters: {
            mfName,
            mfVersion
        },
        distFolder
    });
    const nextSteps = [
        `cd ${targetDir}`,
        `${pkgManager} install`,
        `${pkgManager} run ${"lib" === projectType ? 'mf-dev' : 'dev'}`
    ];
    (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.note)(nextSteps.join('\n'), 'Next steps');
    (0, __WEBPACK_EXTERNAL_MODULE__clack_prompts_3cae1695__.outro)('Done.');
}
const TEMPLATES = [
    'provider-modern',
    'provider-rsbuild',
    'provider-rslib',
    'provider-rslib-storybook',
    'consumer-modern',
    'consumer-rsbuild'
];
create({
    name: 'Module Federation',
    templates: TEMPLATES
});
export { TEMPLATES };
